from time import time

# Bad code!

def animations(animation) -> tuple[list[str], int]:
    wave = ['▁', '▃', '▅', '▇', '█', '▇', '▅', '▃']
    spin = ['⠋', '⠙', '⠹', '⠸', '⠼', '⠴', '⠦', '⠧', '⠇']
    arrows = ['←', '↖', '↑', '↗', '→', '↘', '↓', '↙']
    triangles = ['▲', '▶', '▼', '◀', '▲', '▶', '▼', '◀']
    line = ['|', '/', '—', '\\', '|', '/', '—', '\\']

    order = [wave.copy(), line.copy(), spin.copy(), arrows.copy(), triangles.copy()]
    return order[animation % len(order) - 1], animation % (len(order) + 1)


def main(string, animation) -> str:
    symbols, animation = animations(animation)

    timer = int((time() - int(time())) * len(symbols))

    if animation == 1:
        for _wave in range(3):
            string += symbols[(timer + len(symbols) + _wave % len(symbols)) % len(symbols)]
    else:
        string += f" {symbols[timer]} "

    return string



from sys import stdout
from time import time
from .animations import main


class Bar:
    def __init__(self, total: int, bar_amount: int = 20, animation: int = 1) -> None:
        if total <= 0:
            raise ValueError("The 'total' parameter must be a positive integer greater than zero.")
        if not isinstance(total, int):
            raise ValueError("The 'total' parameter must be an integer.")

        self.done = 0
        self.total = total
        self.amount = bar_amount
        self.animation = animation

        self.start_time = time()
        self.elapsed_time = 0.0
        self.averages = []

        self.max_string_length = 0

    def update(self, done: int) -> bool:
        """
        Updates the progress bar with the given completed tasks.

        :param done: Number of completed tasks.
        :return: True if all tasks are completed, False otherwise.
        """
        self.done = done

        if self.done >= self.total:
            self._output("(!)")
        elif self.animation != 0:
            self._output(main("", self.animation))
        else:
            self._output("...")

        self.start_time = time()
        return self.done >= self.total

    def _output(self, prefix: str) -> None:
        """
        Outputs the progress bar.

        :param prefix: Prefix for the animation.
        """
        filled_bars = int(self.amount * self.done / self.total)

        # Progress bar
        progress_bar = f"{prefix} ["
        progress_bar += "█" * filled_bars
        progress_bar += "-" * (self.amount - filled_bars)
        progress_bar += "] "

        # Progress tracker
        progress_bar += f"{self.done}/{self.total} "
        progress_bar += f"[{self.done / self.total * 100: .1f}%]"

        # Time tracker
        self.elapsed_time = time() - self.start_time
        self.averages.append(self.elapsed_time)
        if self.done < self.total:
            average = sum(self.averages) / len(self.averages)
            progress_bar += f" ({self.elapsed_time: .3f}s, eta: {int(average * (self.total - self.done))}s)"
        else:
            progress_bar += f" (Done in {sum(self.averages): .3f} seconds)"

        # Fix overflow error
        progress_bar += " " * (self.max_string_length - len(progress_bar))
        self.max_string_length = len(progress_bar)

        # Write to the console
        stdout.write(f"\r{progress_bar}")
        stdout.flush()



from setuptools import setup, find_packages
from pathlib import Path


VERSION = "0.5.6"


this_directory = Path(__file__).parent
long_description = (this_directory / "README.md").read_text(encoding='utf-8')

setup(
    name='CoolBar',
    packages=find_packages(),
    version=VERSION,
    license='Apache-2.0',
    description='Dynamic Python Progress Bar',
    long_description=long_description,
    long_description_content_type='text/markdown',
    author='ReloadingBee',
    author_email='reloadingbee@gmail.com',
    url='https://github.com/ReloadingBee/CoolBar',
    download_url=f'https://github.com/ReloadingBee/CoolBar/archive/v_{VERSION}.tar.gz',
    keywords=['progress', 'loading', 'bar', 'dynamic', 'python'],
    install_requires=[],
    classifiers=[
        'Development Status :: 4 - Beta',
        'Intended Audience :: Developers',
        'Topic :: Software Development :: Build Tools',
        'Topic :: Terminals',
        'License :: OSI Approved :: Apache Software License',
        'Programming Language :: Python :: 3',
    ],
)
